﻿Imports System
Imports System.ComponentModel
Imports Autodesk.AutoCAD

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(TestCode_Main.myInitialize))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(TestCode_Main.myCommands))> 




<System.Runtime.InteropServices.ComVisible(False)> _
Public Class TestCode_Main
   Private Shared pEventHandler As myEventHandler = Nothing


   Public Class myInitialize
      Implements Autodesk.AutoCAD.Runtime.IExtensionApplication

      Public Sub Initialize() Implements Autodesk.AutoCAD.Runtime.IExtensionApplication.Initialize
         'entry point
      End Sub

      Public Sub Terminate() Implements Autodesk.AutoCAD.Runtime.IExtensionApplication.Terminate
         Call myCommands.CMDEventsDeActivate()
      End Sub
   End Class




   Public Class myCommands

      ''' <summary>start watching events</summary>
      ''' <remarks></remarks>
      <Autodesk.AutoCAD.Runtime.CommandMethod("EventsActivate", Autodesk.AutoCAD.Runtime.CommandFlags.Redraw Or Autodesk.AutoCAD.Runtime.CommandFlags.UsePickSet Or Autodesk.AutoCAD.Runtime.CommandFlags.Session)> _
      Public Shared Sub CMDEventsActivate()
         Dim tAcadDoc As ApplicationServices.Document = Nothing
         Dim tGoOn As Boolean = True

         'first kill existing events (if there are any)
         Call CMDEventsDeActivate()


         'initialize eventhandler for current doc
         Try
            tAcadDoc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument      'get active document
            If tAcadDoc IsNot Nothing Then
               Dim tEventHandler As myEventHandler = New myEventHandler(tAcadDoc)
               If tEventHandler.isValid Then
                  pEventHandler = tEventHandler
                  AddHandler pEventHandler.ObjectModifications, AddressOf HandleObjectChanges
                  pEventHandler.enabled = True
               End If
            End If
         Catch ex As Exception
            Call MsgBox("Kein Dwg geladen oder AutoCAD nicht ansprechbar")
         End Try
      End Sub

      ''' <summary>ends/disables watching events</summary>
      ''' <remarks></remarks>
      <Autodesk.AutoCAD.Runtime.CommandMethod("EventsDeActivate", Autodesk.AutoCAD.Runtime.CommandFlags.Redraw Or Autodesk.AutoCAD.Runtime.CommandFlags.UsePickSet Or Autodesk.AutoCAD.Runtime.CommandFlags.Session)> _
      Public Shared Sub CMDEventsDeActivate()
         If pEventHandler IsNot Nothing Then
            RemoveHandler pEventHandler.ObjectModifications, AddressOf HandleObjectChanges
            Call pEventHandler.myFinalize()
            pEventHandler = Nothing
         End If
      End Sub


      ''' <summary>HandleObjectChanges gets called after modifications/deletions occured</summary>
      ''' <param name="ObjIDsModified"></param>
      ''' <param name="ObjIDsErased"></param>
      ''' <remarks></remarks>
      Private Shared Sub HandleObjectChanges(ByVal CommandName As String, ByRef ObjIDsModified As DatabaseServices.ObjectIdCollection, ByRef ObjIDsErased As DatabaseServices.ObjectIdCollection)
         Dim tCountModified As Integer = 0
         Dim tCountErased As Integer = 0
         pEventHandler.enabled = False    'to ensure that modifications during this sub does not run into recursive-/endless-mode

         If ObjIDsModified IsNot Nothing Then tCountModified = ObjIDsModified.Count
         If ObjIDsErased IsNot Nothing Then tCountErased = ObjIDsErased.Count

         Call MsgBox(CommandName & ":" & vbNewLine & tCountModified.ToString & " Objekte geändert" & vbNewLine & tCountErased.ToString & " Objekte gelöscht")

         pEventHandler.enabled = True    'resume handling of events 
      End Sub
   End Class

End Class
