﻿Imports Autodesk.AutoCAD



Public Class myEventHandler
   Private pAcadDoc As ApplicationServices.Document = Nothing
   Private pAcadDocDB As DatabaseServices.Database = Nothing
   Private pAcadDocED As EditorInput.Editor = Nothing
   Private pCommandActive As String = ""
   Private pCommandActiveStacked As String = ""
   Private pCommandActiveLast As String = ""

   Private pEnableEvents As Boolean = True

   Private pObjectIDsModified As DatabaseServices.ObjectIdCollection = Nothing
   Private pObjectIDsErased As DatabaseServices.ObjectIdCollection = Nothing


   Public Event ObjectModifications(ByVal CommandName As String, ByRef ObjIDsModified As DatabaseServices.ObjectIdCollection, ByRef ObjIDsErased As DatabaseServices.ObjectIdCollection)



   Public Sub New(ByRef AcadDoc As ApplicationServices.Document)
      If AcadDoc IsNot Nothing Then
         'initialize eventhandler
         pAcadDoc = AcadDoc
         pAcadDocDB = pAcadDoc.Database
         pAcadDocED = pAcadDoc.Editor
         'command-start-end
         AddHandler pAcadDoc.CommandWillStart, AddressOf CommandStarted
         AddHandler pAcadDoc.CommandEnded, AddressOf CommandEnded
         'object-modified-erased
         AddHandler pAcadDocDB.ObjectModified, AddressOf ObjectModified
         AddHandler pAcadDocDB.ObjectErased, AddressOf ObjectErased
         'editor-quiescent
         AddHandler pAcadDocED.EnteringQuiescentState, AddressOf AcadGotQuiescent
      End If
   End Sub
   Public Sub myFinalize()
      If pAcadDoc IsNot Nothing Then
         'command-start-end
         RemoveHandler pAcadDoc.CommandWillStart, AddressOf CommandStarted
         RemoveHandler pAcadDoc.CommandEnded, AddressOf CommandEnded
         'object-modified-erased
         RemoveHandler pAcadDocDB.ObjectModified, AddressOf ObjectModified
         RemoveHandler pAcadDocDB.ObjectErased, AddressOf ObjectErased
         'editor-quiescent
         RemoveHandler pAcadDocED.EnteringQuiescentState, AddressOf AcadGotQuiescent
         pAcadDoc = Nothing
         pAcadDocDB = Nothing
         pAcadDocED = Nothing
      End If
   End Sub


   Public ReadOnly Property isValid() As Boolean
      Get
         Return (pAcadDoc IsNot Nothing)
      End Get
   End Property
   Public Property enabled() As Boolean
      Get
         Return pEnableEvents
      End Get
      Set(ByVal value As Boolean)
         pEnableEvents = value
      End Set
   End Property



   ''' <summary>watch 'command-start'</summary>
   ''' <param name="sender"></param>
   ''' <param name="e"></param>
   ''' <remarks></remarks>
   Private Sub CommandStarted(ByVal sender As Object, ByVal e As ApplicationServices.CommandEventArgs)
      If pCommandActive = "" Then   'just to make sure we don't get stacked-commands (like transparent zoom or _undo, ...)
         pCommandActive = e.GlobalCommandName
         pCommandActiveLast = e.GlobalCommandName
      Else
         pCommandActiveStacked = e.GlobalCommandName
      End If
   End Sub
   ''' <summary>watch 'command-end'</summary>
   ''' <param name="sender"></param>
   ''' <param name="e"></param>
   ''' <remarks></remarks>
   Private Sub CommandEnded(ByVal sender As Object, ByVal e As ApplicationServices.CommandEventArgs)
      If pCommandActive = e.GlobalCommandName Then 'just to make sure we only come back from top-command (and not from stacked command)
         pCommandActive = ""
      ElseIf pCommandActiveStacked = e.GlobalCommandName Then
         pCommandActiveStacked = ""
      End If
   End Sub

   Private Sub ObjectModified(ByVal sender As Object, ByVal e As DatabaseServices.ObjectEventArgs)
      If pEnableEvents Then
         If pObjectIDsModified Is Nothing Then pObjectIDsModified = New DatabaseServices.ObjectIdCollection 'then we have to initialize the collection
         If Not pObjectIDsModified.Contains(e.DBObject.ObjectId) Then pObjectIDsModified.Add(e.DBObject.ObjectId) 'add objectid
      End If
   End Sub
   Private Sub ObjectErased(ByVal sender As Object, ByVal e As DatabaseServices.ObjectErasedEventArgs)
      If pEnableEvents Then
         If pObjectIDsErased Is Nothing Then pObjectIDsErased = New DatabaseServices.ObjectIdCollection 'then we have to initialize the collection
         If Not pObjectIDsErased.Contains(e.DBObject.ObjectId) Then pObjectIDsErased.Add(e.DBObject.ObjectId) 'add objectid
      End If
   End Sub


   Private Sub AcadGotQuiescent(ByVal sender As Object, ByVal e As System.EventArgs)
      If pEnableEvents AndAlso ((pObjectIDsErased IsNot Nothing) OrElse (pObjectIDsModified IsNot Nothing)) Then
         'raise event to get inform upper process
         RaiseEvent ObjectModifications(pCommandActiveLast, pObjectIDsModified, pObjectIDsErased)
         'cleanup collections
         If pObjectIDsModified IsNot Nothing Then pObjectIDsModified = Nothing
         If pObjectIDsErased IsNot Nothing Then pObjectIDsErased = Nothing
      End If
   End Sub

End Class
